from django.core.exceptions import ValidationError
from django.test import TestCase

from drip.models import Drip, QuerySetRule


class RulesTestCase(TestCase):
    def setUp(self):
        self.drip = Drip.objects.create(
            name="A Drip just for Rules",
            subject_template="Hello",
            body_html_template="KETTEHS ROCK!",
        )

    def test_valid_rule(self):
        rule = QuerySetRule(
            drip=self.drip,
            field_name="date_joined",
            lookup_type="lte",
            field_value="now-60 days",
        )
        rule.clean()

    def test_bad_field_name(self):
        rule = QuerySetRule(
            drip=self.drip,
            field_name="date__joined",
            lookup_type="lte",
            field_value="now-60 days",
        )
        self.assertRaises(ValidationError, rule.clean)

    def test_bad_field_value(self):
        rule = QuerySetRule(
            drip=self.drip,
            field_name="date_joined",
            lookup_type="lte",
            field_value="now-2 months",
        )
        self.assertRaises(ValidationError, rule.clean)



from apscheduler.schedulers.background import BackgroundScheduler
from django.conf import settings
from django.core.management import call_command

DRIP_SCHEDULE_SETTINGS = getattr(settings, "DRIP_SCHEDULE_SETTINGS", {})


DRIP_SCHEDULE = DRIP_SCHEDULE_SETTINGS.get("DRIP_SCHEDULE", False)
DRIP_SCHEDULE_DAY_OF_WEEK = DRIP_SCHEDULE_SETTINGS.get("DRIP_SCHEDULE_DAY_OF_WEEK", 0)
DRIP_SCHEDULE_HOUR = DRIP_SCHEDULE_SETTINGS.get("DRIP_SCHEDULE_HOUR", 0)
DRIP_SCHEDULE_MINUTE = DRIP_SCHEDULE_SETTINGS.get("DRIP_SCHEDULE_MINUTE", 0)


def cron_send_drips():
    def call_send_drips_command():
        call_command("send_drips")

    if DRIP_SCHEDULE:
        cron_scheduler = BackgroundScheduler()
        cron_scheduler.add_job(
            call_send_drips_command,
            "cron",
            day_of_week=DRIP_SCHEDULE_DAY_OF_WEEK,
            hour=DRIP_SCHEDULE_HOUR,
            minute=DRIP_SCHEDULE_MINUTE,
            timezone=getattr(settings, "TIME_ZONE", "UTC"),
        )
        cron_scheduler.start()



"""Defines the types to be used within
the type annotations all along the system"""
from datetime import timedelta
from typing import TypeVar, Union

from django.db.models import F

AbstractQuerySetRule = TypeVar("AbstractQuerySetRule")
TimeDeltaOrStr = Union[timedelta, str]
BoolOrStr = Union[bool, str]
FExpressionOrStr = Union[F, str]
FieldValue = Union[TimeDeltaOrStr, BoolOrStr, FExpressionOrStr]
